<?php
// $Id: luceneapi_node.facet.inc,v 1.3 2009/01/22 03:41:23 cpliakas Exp $

/**
 * @file
 * Facet queries for the Search Lucene Content module.
 */

/**
 * Appends a query for the content type facet.
 *
 * @param $query
 *   A Zend_Search_Lucene_Search_Query_Boolean object containing the facet queries.
 * @return
 *   NULL
 */
function _luceneapi_node_append_type_query(Zend_Search_Lucene_Search_Query_Boolean $query) {
  $types = luceneapi_facet_value('type', array());
  if (is_array($types) && !empty($types)) {
    $type_query = luceneapi_query_multiterm();
    foreach ($types as $type) {
      if ($type) {
        $type_query->addTerm(luceneapi_term_get($type, 'type'));
      }
    }
    if (count($type_query->getTerms())) {
      $query->addSubquery($type_query, TRUE);
    }
  }
}

/**
 * Appends query for the taxonomy facet.
 *
 * @param $query
 *   A Zend_Search_Lucene_Search_Query_Boolean object containing the facet queries.
 * @return
 *   NULL
 */
function _luceneapi_node_append_taxonomy_query(Zend_Search_Lucene_Search_Query_Boolean $query) {
  $terms = luceneapi_facet_value('category', array());
  if (!empty($terms)) {
    foreach ($terms as $term_id) {
      $query->addSubquery(luceneapi_query_term($term_id, 'category'), TRUE);
    }
  }
}

/**
 * Appends query for the node arthors.
 *
 * @param $query
 *   A Zend_Search_Lucene_Search_Query_Boolean object containing the facet queries.
 * @return
 *   NULL
 * @todo tighten up the parsing, figure out spaces in users
 */
function _luceneapi_node_append_user_query(Zend_Search_Lucene_Search_Query_Boolean $query) {
  $users = luceneapi_facet_value('author', array());
  if (!empty($users)) {
    $author_query = luceneapi_query_boolean();
    $parsed = array_map('trim', explode(',', $users));
    foreach ($parsed as $name) {
      if (strlen($name)) {
        $wcard = luceneapi_query_wildcard(strtolower($name), 'name');
        $author_query->addSubquery($wcard, TRUE);
      }
    }
    if (count($author_query->getSubqueries())) {
      $query->addSubquery($author_query, TRUE);
    }
  }
}

/**
 * Appends query for changed after facet.
 *
 * @param $query
 *   A Zend_Search_Lucene_Search_Query_Boolean object containing the facet queries.
 * @return
 *   NULL
 * @see _luceneapi_node_append_date_subquery()
 */
function _luceneapi_node_append_changed_query(Zend_Search_Lucene_Search_Query_Boolean $query) {
  _luceneapi_node_append_date_subquery('changed', $query);
}

/**
 * Appends query for created after facet.
 *
 * @param $query
 *   A Zend_Search_Lucene_Search_Query_Boolean object containing the facet queries.
 * @return
 *   NULL
 * @see _luceneapi_node_append_date_subquery()
 */
function _luceneapi_node_append_created_query(Zend_Search_Lucene_Search_Query_Boolean $query) {
  _luceneapi_node_append_date_subquery('created', $query);
}

/**
 * Provides common functionality for date facets, appends facets.
 *
 * @param $field
 *   A string containing the name of the search field that corresponds with the
 *   name of the index field being searched.
 * @param $query
 *   A Zend_Search_Lucene_Search_Query_Boolean object containing the facet queries.
 * @return
 *   NULL
 */
function _luceneapi_node_append_date_subquery($field, Zend_Search_Lucene_Search_Query_Boolean $query) {
  $usertime = luceneapi_facet_value($field, '');
  if (!empty($usertime)) {
    if (date_is_valid($usertime, DATE_DATETIME)) {
      $unixtime = date_convert($usertime, DATE_DATETIME, DATE_UNIX);
      $query->addSubquery(luceneapi_query_range($unixtime, time(), FALSE, $field), TRUE);
    }
    else {
      form_set_error($field, sprintf('Date not valid for %s field', $field));
      return;
    }
  }
}
