<?php
// $Id: luceneapi.admin.inc,v 1.11.2.4 2009/02/27 03:37:45 cpliakas Exp $

/**
 * @file
 * Administrative settings callbacks for the Search Lucene API module.
 */

/**
 * Administrative settings.
 *
 * @return
 *   An array of form elements.
 */
function luceneapi_admin_settings() {
  $form = array();

  // index settings fieldset
  $form['global'] = array(
    '#type' => 'fieldset',
    '#title' => t('Global settings'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
    '#description' => t(
      'Global search settings for all Search Lucene API modules'
    ),
  );

  // builds array of Search Lucene API modules
  $files = module_rebuild_cache();
  $hijack_options = array('0' => 'None');
  foreach (luceneapi_module_list() as $module) {
    $hijack_options[$module] = $files[$module]->info['name'];
  }

  // determines which Search Lucene API module will hijack the search box
  $form['global']['luceneapi_default_search'] = array(
    '#type' => 'radios',
    '#title' => t('Hijack search box'),
    '#default_value' => luceneapi_setting_get('default_search'),
    '#options' => $hijack_options,
    '#description' => t(
      'Determine which module will take over the default search box'
    ),
  );

  // result set limitation to prevent excessive score calculations
  $form['global']['luceneapi_resultset_limit'] = array(
    '#type' => 'select',
    '#title' => t('Result set limit'),
    '#default_value' => luceneapi_setting_get('resultset_limit'),
    '#options' => drupal_map_assoc(array(0, 500, 1000, 2000, 5000, 10000)),
    '#description' => t(
      'Limit score calculation for extremely large result sets (0 = no limit).'
    ),
  );

  // description for error fieldset
  $description = 'Determines how to display application level errors, such as'
               .' the exceptions thrown by Zend Framework components.';

  // error handling field set
  $form['error'] = array(
    '#type' => 'fieldset',
    '#title' => t('Error settings'),
    '#collapsible' => TRUE,
    '#description' => t($description),
  );

  // how errors are displayed, if they are displayed at all
  $form['error']['luceneapi_error_mode'] = array(
    '#type' => 'radios',
    '#title' => t('Error mode'),
    '#default_value' => luceneapi_setting_get('error_mode'),
    '#options' => array(
      LUCENEAPI_ERROR_DISPLAY_DEBUG => t('Debug: detailed messages'),
      LUCENEAPI_ERROR_DISPLAY_GENERIC => t('Generic: nonspecific message'),
      LUCENEAPI_ERROR_DISPLAY_NONE => t('None: no messages displayed'),
    ),
    '#description' => t(
      'These errors should not be displayed in production environments.'
    ),
  );

  // the default error message when general mode is enabled
  $form['error']['luceneapi_error_message'] = array(
    '#type' => 'textfield',
    '#title' => t('Generic error message'),
    '#default_value' => luceneapi_setting_get('error_message'),
    '#maxlength' => 255,
    '#description' => t(
      'Error that will be displayed if <b>Generic</b> mode is selected.'
    ),
  );

  return system_settings_form($form);
}

/**
 * Returns array of settings common to Search Lucene API modules that define a
 * Lucene index.  Adds a submit handler to clear Search Lucene API cached
 * items.
 *
 * @param $module
 *   A string containing the module the settings are being applied to.
 * @param &$form
 *   An array containing the admin settings form.
 * @return
 *   An array of form elements.
 */
function luceneapi_common_settings($module, array &$form) {

  // adds submit handler to clear cached items
  $form['#validate']['luceneapi_admin_form_validate'] = array();

  // inititalizes common form elements
  $common = array();

  // passes module name
  $common['module'] = array(
    '#type' => 'hidden',
    '#value' => $module,
  );

  // index settings fieldset
  $common['index'] = array(
    '#type' => 'fieldset',
    '#title' => t('Index settings'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );

  // path to index directory
  $common['index']["{$module}_index_path"] = array(
    '#type' => 'textfield',
    '#title' => t('Index path'),
    '#default_value' => luceneapi_variable_get($module, 'index_path'),
    '#maxlength' => 255,
    '#description' => t('A file system path where Lucene index files are stored.'),
    '#after_build' => array('system_check_directory'),
  );

  // index update throttle
  $common['index']["{$module}_update_limit"] = array(
    '#type' => 'select',
    '#title' => t('Items to index per update'),
    '#default_value' => luceneapi_variable_get($module, 'update_limit'),
    '#options' => drupal_map_assoc(array(10, 20, 50, 100, 200, 500, 1000, 2000, 5000, 10000)),
    '#description' => t('The maximum number of items that will be indexed per update.'),
  );

  // whether or not updates are allowed via cron
  $common['index']["{$module}_cron_updates"] = array(
    '#type' => 'radios',
    '#title' => t('Update index via cron'),
    '#default_value' => luceneapi_variable_get($module, 'cron_updates'),
    '#options' => array(
      1 => t('Yes'),
      0 => t('No'),
    ),
    '#description' => t(
      'Set to "No" if you want to manage the index solely via Drush.'
    ),
  );

  // optimize the index after it is updated
  $common['index']["{$module}_optimize_on_update"] = array(
    '#type' => 'radios',
    '#title' => t('Optimize on update'),
    '#default_value' => luceneapi_variable_get($module, 'optimize_on_update'),
    '#options' => array(
      1 => t('Yes'),
      0 => t('No'),
    ),
    '#description' => t(
      'Setting to yes will automatically optimize the index after it is updated.'
    ),
  );

  // option to optimize the index
  $common['index']['optimize'] = array(
    '#type' => 'submit',
    '#value' => t('Optimize index'),
  );

  // index settings fieldset
  $common['search'] = array(
    '#type' => 'fieldset',
    '#title' => t('Search page settings'),
    '#collapsible' => TRUE,
  );

  // default results per page for all modules that implement SLAPI
  $common['search']["{$module}_results_per_page"] = array(
    '#type' => 'select',
    '#title' => t('Results per page'),
    '#default_value' => luceneapi_variable_get($module, 'results_per_page'),
    '#options' => drupal_map_assoc(array(5, 10, 20, 30, 50, 100)),
    '#description' => t('Number of items that will be shown on the search results page.'),
  );

  // index settings fieldset
  $common['cache'] = array(
    '#type' => 'fieldset',
    '#title' => t('Cache settings'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );

  // cache expensive operations
  $common['cache']["{$module}_caching_enabled"] = array(
    '#type' => 'radios',
    '#title' => t('Enable caching'),
    '#default_value' => luceneapi_variable_get($module, 'caching_enabled'),
    '#options' => array(
      1 => t('Yes'),
      0 => t('No'),
    ),
    '#description' => t(
      'Cache search results and other intensive items.'
    ),
  );

  // default results per page for all modules that implement SLAPI
  $common['cache']["{$module}_number_cached"] = array(
    '#type' => 'select',
    '#title' => t('Maximum size of cached result set'),
    '#default_value' => luceneapi_variable_get($module, 'number_cached'),
    '#options' => drupal_map_assoc(array(100, 200, 500, 1000, 2000, 5000, 10000)),
    '#description' => t(
      'The first <i>n</i> number of results in a result set will be cached.'
    ),
  );

  // option to clear the search results
  $common['cache']['clear'] = array(
    '#type' => 'submit',
    '#value' => t('Clear search results cache'),
  );

  // description for faceted search setting
  $facets_descrip = 'Setting  to <em>No</em> will force a return value of FALSE when'
                  .' invoking <code>hook_luceneapi_index(&#039;facets&#039;)</code>.';

  // gets enabled facets, facets available to $module
  $options = array();
  $type = module_invoke($module, 'luceneapi_index', 'type');
  $enabled_facets = luceneapi_variable_get($module, 'enabled_facets');
  foreach (luceneapi_facet_module_list() as $facet_module) {
    if ($names = _luceneapi_facet_names($facet_module, $module, $type)) {
      if (!is_array($names)) {
        $names = array($names);
      }
      foreach ($names as $name) {
        // normalizes facet name, formats options array
        $facet_name = _luceneapi_facet_normalize_name($facet_module, $name);
        $options[$facet_name] = $name;
        // facets are enabled by default, so adds setting if it is not set
        if (!array_key_exists($facet_name, $enabled_facets)) {
          $enabled_facets[$facet_name] = $facet_name;
        }
      }
    }
  }

  // if there are options, adds them to the form
  if (!empty($options)) {
    $common['facets'] = array(
      '#type' => 'fieldset',
      '#title' => t('Facet settings'),
      '#collapsible' => TRUE,
      '#description' => t('Configure which facets are allowed for this module.'),
    );
    // whether or not faceted searches are allowed
    $common['facets']["{$module}_facets_enabled"] = array(
      '#type' => 'radios',
      '#title' => t('Enable faceted searches'),
      '#default_value' => luceneapi_variable_get($module, 'facets_enabled'),
      '#options' => array(
        1 => t('Yes'),
        0 => t('No'),
      ),
      '#description' => t($facets_descrip),
    );
    // disable facets on a per facet basis
    $common['facets']["{$module}_enabled_facets"] = array(
      '#type' => 'checkboxes',
      '#title' => t('Enable facets'),
      '#prefix' => '<div class="criterion">',
      '#suffix' => '</div>',
      '#options' => $options,
      '#default_value' => $enabled_facets,
    );
  }

  // appends common form elements to settings form
  $form = array_merge_recursive($form, $common);
}

/**
 * Validate function for admin forms, clears luceneapi cache items.
 *
 * @param $form_id
 *   A string containing the ID of form being validated.
 * @param $form_values
 *   An flattened array of form values.
 * @param $form
 *   A FAPI array modeling the submitted form.
 * @return
 *   NULL
 */
function luceneapi_admin_form_validate($form_id, $form_values, $form) {
  if (!in_array($form_values['module'], luceneapi_module_list())) {
    form_set_error('module', sprintf(
      '%s is not a valid Search Lucene API module', $form_values['module']
    ));
    return;
  }
  if ($form_values['op'] == t('Optimize index')) {
    drupal_goto(sprintf('admin/settings/%s/optimize', $form_values['module']));
  }
  if ($form_values['op'] == t('Clear search results cache')) {
    drupal_goto(sprintf('admin/settings/%s/cache', $form_values['module']));
  }

  // clears items in cache table
  luceneapi_clear_cache();
}

/**
 * Confirmation form for admin tasks.
 *
 * @param $module
 *   A string containing the module managing the index being optimized.
 * @param $task
 *   A string containing the task type, i.e. optimize
 * @return
 *   NULL
 */
function luceneapi_admin_confirm($module, $task) {
  $return_path = sprintf('admin/settings/%s', $module);
  switch ($task) {
    case 'optimize':
      $button_text = 'Optimize index';
      $confim_message = 'Are you sure you want to optimize the index?';
      $description = 'A Lucene index consists of many segments. Each segment is'
                   .' a completely independent set of data. New documents are'
                   .' added to the index by creating new segment. Increasing'
                   .' number of segments reduces quality of the index, but'
                   .' index optimization restores it.  This process may take a'
                   .' while for larger indexes, so please be patient.';
      break;
    case 'cache':
      $button_text = 'Clear cache';
      $confim_message = 'Are you sure you want to clear the results cache?';
      $description = 'Maintaing a search results cache can reduce the time it'
                   .' takes to search and sort content, but it can become very'
                   .' large over time.  It may also store results that are'
                   .' out of date or are no longer valid';
      break;
    default:
      drupal_goto($return_path);
      break;
  }

  // returns the confirmation form
  return confirm_form(
    array(
      'module' => array(
        '#type' => 'hidden',
        '#value' => $module,
      ),
      'task' => array(
        '#type' => 'hidden',
        '#value' => $task,
      ),
    ),
    t($confim_message),
    $return_path,
    t($description),
    t($button_text),
    t('Cancel')
  );
}

/**
 * Validate function for the admin functions that require confirmation such as
 * clearing the cache and index optimization.
 *
 * @param $form_id
 *   A string containing the ID of form being validated.
 * @param $form_values
 *   An flattened array of form values.
 * @param $form
 *   A FAPI array modeling the submitted form.
 * @return
 *   NULL
 */
function luceneapi_admin_confirm_validate($form_id, $form_values, $form) {
  if (!in_array($form_values['module'], luceneapi_module_list())) {
    form_set_error('module', sprintf(
      '%s is not a valid Search Lucene API module', $form_values['module']
    ));
  }
  $valid_tasks = array('optimize', 'cache');
  if (!in_array($form_values['task'], $valid_tasks)) {
    form_set_error('task', sprintf(
      '%s is not a valid task', $form_values['task']
    ));
  }
}

/**
 * Submit handler for the admin functions that require confirmation such as
 * clearing the cache and index optimization.
 *
 * @param $form_id
 *   A string containing the ID of form being validated.
 * @param $form
 *   A FAPI array modeling the submitted form.
 * @return string
 *   A string containing the path to the modules admin settings page.
 */
function luceneapi_admin_confirm_submit($form_id, &$form) {
  if ($form['confirm']) {
    switch ($form['task']) {
      case 'optimize':
        if (luceneapi_optimize_index($form['module'])) {
          drupal_set_message('Index sucessfully optimized');
        }
        break;
      case 'cache':
        cache_clear_all($form['module'], LUCENEAPI_CACHE_TABLE, TRUE);
        drupal_set_message(
          sprintf('Cached %s search results cleared', $form['module'])
        );
        break;
    }
    return 'admin/settings/'. $form['module'];
  }
}
