<?php
// $Id: luceneapi.hooks.inc,v 1.1.4.2 2009/02/25 01:22:27 cpliakas Exp $

/**
 * @file
 * Example file that defines the Search Lucene API hooks.
 */

// this file is an example only and not meant to be sourced
return;

/**
 * @defgroup hooks Search Lucene API hooks.
 * @{
 */

/**
 * Alerts Search Lucene API that this module manages a Lucene index.  Modules
 * that implement hook_luceneapi_index() should also implement the core search
 * hooks hook_search() and hook_update_index().
 *
 * @param $op
 *   A string containing the operation being performed.
 * @return
 *   Varies dependent on $op:
 *    - path: A string containing the filesystem path to the Lucene index. It is
 *        possible for multiple modules to manage the same index.
 *    - type: A string containing the type of content being indexed.
 *    - use facets: A boolean flagging whether to use facets, usually the return
 *        of user_access('use advanced search').
 * @see hook_search()
 * @see hook_update_index()
 */
function hook_luceneapi_index($op) {
}

/**
 * Allows modules to alter the search query object. This hook is invoked after
 * the facet callbacks are invoked and before the search is executed.. The
 * query passed to this hook is a container of both the form query and facet
 * queries.
 *
 * @param $query
 *   A Zend_Search_Lucene_Search_Query_Boolean object modeling the search query.
 * @param $module
 *   A string containing the module handling the search.
 * @param $type
 *   A string containing the type of content $module indexes, NULL if no type.
 * @return
 *   NULL
 */
function hook_luceneapi_query_alter($query, $module, $type = NULL) {
}

/**
 * Enables modules to log and process the positive keys found during a search.
 *
 * @param $positive_keys
 *   An array of positive keys found during the search.
 * @param $module
 *   A string containing the module defining the Lucene index.
 * @param $type
 *   A string contaning the type of content being indexed by $module.
 * @return
 *   NULL
 */
function hook_luceneapi_positive_keys($positive_keys, $module, $type = NULL) {
}

/**
 * Allows modules to alter the item array for each search result.  For example,
 * you could hook into Search Lucene Content to modify the titles and links of
 * the nodes being displayed in the result set.
 *
 * @param &$result
 *   An array containing a search result item.  The array is passed by
 *   reference, so modifying the passed array will change what is displayed on
 *   the search results page.
 * @param $module
 *   A string containing the module handling the search.
 * @param $type
 *   A string contining the type of content being indexed by $module.
 * @return
 *   NULL
 */
function hook_luceneapi_result_alter(&$result, $module, $type = NULL) {
}

/**
 * Allows modules to alter the document object being added to the index. This
 * hook is usually implemented to add additional identifiers to the document.
 *
 * @param $doc
 *   A Zend_Search_Lucene_Document object being added to the index.
 * @param $module
 *   A string containing the module defining the index being updated.
 * @param $type
 *   A string contining the type of content being indexed by $module.
 * @return
 *   NULL
 */
function hook_luceneapi_document_alter($doc, $module, $type = NULL) {
}

/**
 * Allows modules to add subqueries limiting which documents are removed from
 * Lucene indexes.  For example, to only delete nodes from the index of the
 * content type page, pass a query that searches for nodes of the page content
 * type.
 *
 * @param $item
 *   A mixed value modeling the content being removed from the index, usually a
 *   drupal node object.
 * @param $module
 *   A string containing the module defining the Lucene index.
 * @param $type
 *   A string contining the type of content being indexed by $module.
 * @return
 *   A Zend_Search_Lucene_Search_Query object.
 */
function hook_luceneapi_document_delete($item, $module, $type = NULL) {
}

/**
 * Invoked by modules that define one or more facets for Search Lucene API
 * modules.
 *
 * @param $op
 *   A string containing the operation being performed.
 * @param $module
 *   A string containing the module handling the search.
 * @param $type
 *   A string containing the type of content $module indexes, NULL if no type.
 * @return
 *   Varies dependent on $op:
 *     - name: An array of facet names or a string containing a facet name.
 *     - facet: An array of form elements building the facets.
 *     - callback: An array of callbacks or a string containing a callback.
 */
function hook_luceneapi_facet($op, $module, $type = NULL) {
}

/**
 * @} End of "defgroup hooks".
 */
